import usb
import time

RQ_GAS_READ = 1
RQ_ALARM_READ = 2

class MCUBoard:
    """
    Generic class for accessing McuBoard.
    """

    def find_devices(self):
        boards = [dev for bus in usb.busses()
                      for dev in bus.devices
                      if (dev.idVendor, dev.idProduct) == (0x16c0, 0x05dc)]
        return boards
    
    def __init__(self, board_num: int = 0):
        try:
            self.device = self.find_devices()[0]
            self.handle = self.device.open()
            self.board_num = board_num
        except IndexError:
            print("No USB Devices Found")
            raise Exception

    def re_init(self):
        try:
            self.device = self.find_devices()[0]
            self.handle = self.device.open()
        except IndexError:
            print("No USB Devices Found")

    def usb_write(self, request, data=[], index=0, value=0):
        """
        Send data output to usb.
        """
        reqType = usb.TYPE_VENDOR | usb.RECIP_DEVICE | usb.ENDPOINT_OUT
        self.handle.controlMsg(
            reqType, request, data, value=value, index=index
        )
    
    def usb_read(self, request, length=1, index=0, value=0):
        """
        Request data input from usb.
        """

        reqType = usb.TYPE_VENDOR | usb.RECIP_DEVICE | usb.ENDPOINT_IN
        buf = self.handle.controlMsg(
            reqType, request, length, value=value, index=index
        )
        return buf

class Control:

    def __init__(self):
        self.mcu = MCUBoard()
    
    def re_init(self):
        self.mcu.re_init()

    def read_gas(self, silent: bool = True):
        """
        Read gas sensor value (decimal ratio of floor(rs/r0 * 10))
        """
        read_success = False
        while not read_success:
            try:
                if not silent: print("[USB] Reading Gas Value...", end='')
                read = self.mcu.usb_read(request=RQ_GAS_READ, length=1, index=0, value=0)
                read_success = True
                if not silent: print("Success: {}".format(read[0]))
            except usb.core.USBError:
                if not silent: print("Fail...Retrying")
                self.re_init()
                time.sleep(0.25)
                continue

        return read[0]

    def set_alarm(self, state):
        if not state in [0, 1, 2]:
            print("Alarm has state from 0 - 2.")
            return 1

        try:
            self.mcu.usb_write(request=2, value=state)
        except usb.core.USBError:
            print("[USB] Fail to set alarm state.")
            self.re_init()
            return 1
        return 0

    def get_alarm(self):
        timeout = 3
        while timeout >= 0:
            try:
                alarm_state = self.mcu.usb_read(request=3)
                return alarm_state[0]
            except usb.core.USBError:
                print("[USB] Fail to get alarm state.")
                self.re_init()
                timeout = timeout - 1
                time.sleep(0.05)
        return -1
